from clandestino_interfaces import AbstractMigration

from clandestino_sqlite.infra import SQLiteInfra


class Migration(AbstractMigration):

    infra = SQLiteInfra()

    async def up(self) -> None:
        """Do modifications in database"""
        pass

    async def down(self) -> None:
        """Undo modifications in database"""
        pass



import os
import sqlite3

from contextlib import asynccontextmanager
from sqlite3 import Connection, Cursor

from decouple import AutoConfig

config = AutoConfig(search_path=os.getcwd())


class SQLiteInfra:

    @classmethod
    async def __get_connection(cls) -> Connection:
        str_connection = config("CLANDESTINO_SQLITE_DB_PATH")
        _connection = sqlite3.connect(str_connection)
        return _connection

    @classmethod
    @asynccontextmanager
    async def get_cursor(cls) -> Cursor:
        connection = None
        cursor = None
        try:
            connection = await cls.__get_connection()
            cursor = connection.cursor()
            yield cursor
        except Exception as e:
            print(f"{cls.__class__}::get_cursor")
            raise e
        finally:
            if connection:
                connection.commit()
                if cursor:
                    cursor.close()
                connection.close()



from .infra import SQLiteInfra

from clandestino_interfaces import IMigrateRepository


class SQLiteMigrateRepository(IMigrateRepository, SQLiteInfra):

    @classmethod
    def get_control_table(cls):
        return "clandestino"

    @classmethod
    async def create_control_table(cls) -> None:
        control_table = cls.get_control_table()
        sql = f"""
            CREATE TABLE IF NOT EXISTS {control_table} (
                id INTEGER PRIMARY KEY AUTOINCREMENT,
                migration_name TEXT UNIQUE NOT NULL,
                created_on TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP
            )
        """
        async with cls.get_cursor() as cursor:
            await cursor.execute(sql)

    @classmethod
    async def control_table_exists(cls) -> bool:
        control_table = cls.get_control_table()
        sql = f"""
            SELECT name FROM sqlite_master
            WHERE type='table' AND name=?
        """
        async with cls.get_cursor() as cursor:
            await cursor.execute(sql, [control_table])
            result = await cursor.fetchone()
        return bool(result)

    @classmethod
    async def register_migration_execution(cls, migration_name: str) -> None:
        control_table = cls.get_control_table()
        sql = f"""
            INSERT INTO {control_table} (migration_name)
            VALUES (?)
        """
        async with cls.get_cursor() as cursor:
            await cursor.execute(sql, [migration_name])

    @classmethod
    async def remove_migration_execution(cls, migration_name: str) -> None:
        control_table = cls.get_control_table()
        sql = f"""
            DELETE FROM {control_table} WHERE migration_name = ?
        """
        async with cls.get_cursor() as cursor:
            await cursor.execute(sql, [migration_name])

    @classmethod
    async def migration_already_executed(cls, migration_name: str) -> bool:
        control_table = cls.get_control_table()
        sql = f"""
            SELECT EXISTS (
                SELECT 1 FROM {control_table}
                WHERE migration_name = ?
            )
        """
        async with cls.get_cursor() as cursor:
            await cursor.execute(sql, [migration_name])
            result = await cursor.fetchone()
        return bool(result[0])



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['clandestino_sqlite']

package_data = \
{'': ['*']}

install_requires = \
['clandestino-interfaces>=0.1.0,<0.2.0', 'python-decouple>=3.8,<4.0']

setup_kwargs = {
    'name': 'clandestino-sqlite',
    'version': '0.1.2',
    'description': 'Clandestino SQLite implementation',
    'long_description': '# Clandestino Sqlite\n\nMain project [here](https://github.com/CenturyBoys/clandestino)\n\nThis project uses native python [sqlite3](https://docs.python.org/3/library/sqlite3.html).\n',
    'author': 'XimitGaia',
    'author_email': 'im.ximit@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.11,<4.0',
}


setup(**setup_kwargs)
